﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Web;
using DotNetNuke.Common.Lists;
using DotNetNuke.Entities.Portals;
using DotNetNuke.Entities.Profile;
using DotNetNuke.Entities.Users;
using DotNetNuke.Security.Membership;
using DotNetNuke.Security.Roles;
using DotNetNuke.UI.Skins.Controls;
using DotNetNuke.Entities.Host;
using DotNetNuke.Common.Utilities;
using DotNetNuke.Web.UI.WebControls;
using System.Web.UI;

namespace DotNetNuclear.Modules.Registration.Components
{
    public class DnnMembershipUtilities
    {
        private static DateTime ExpireCookie = DateTime.Now.AddYears(-10);
        private static DateTime PersistCookie = DateTime.Now.AddYears(10);

        /// <summary>
        /// ValidateDNNUser
        /// </summary>
        /// <param name="portalid"></param>
        /// <param name="username"></param>
        /// <param name="password"></param>
        /// <param name="ipaddress"></param>
        /// <param name="user"></param>
        /// <returns></returns>
        public static UserLoginStatus ValidateDNNUser(int portalid, string username, string password, string ipaddress, ref UserInfo user)
        {
            //Try and Log User into DNN
            UserLoginStatus loginStatus = UserLoginStatus.LOGIN_FAILURE;
            user = UserController.ValidateUser(portalid, username, password, "", "", ipaddress, ref loginStatus);

            return loginStatus;
        }

        /// <summary>
        /// CreateSSOUser
        /// </summary>
        /// <param name="portalId"></param>
        /// <param name="newUser"></param>
        /// <param name="roles"></param>
        /// <returns></returns>
        public static UserCreateStatus CreateDnnUser(int portalId, ref UserInfo newUser, List<string> roles)
        {
            UserCreateStatus userCreatedStatus = UserCreateStatus.UnexpectedError;
            try
            {
                newUser.PortalID = portalId;
                newUser.IsSuperUser = false;
                if (String.IsNullOrEmpty(newUser.DisplayName))
                {
                    newUser.DisplayName = String.Format("{0} {1}", newUser.FirstName, newUser.LastName);
                }

                var pCt = new PortalController();
                var portalInf = pCt.GetPortal(portalId);
                PortalSettings pset = PortalController.GetCurrentPortalSettings();

                //Fill MINIMUM Profile Items
                newUser.Profile.PreferredLocale = portalInf.DefaultLanguage;
                newUser.Profile.FirstName = newUser.FirstName;
                newUser.Profile.LastName = newUser.LastName;

                //Set Membership
                if (newUser.Membership == null || String.IsNullOrEmpty(newUser.Membership.Password))
                {
                    UserMembership oNewMembership = new UserMembership(newUser) { Password = UserController.GeneratePassword() };

                    //Bind membership to user
                    newUser.Membership = oNewMembership;
                }
                newUser.Membership.Approved = (pset.UserRegistration == 2);
                newUser.Membership.CreatedDate = System.DateTime.Now;

                //Add the user, ensure it was successful
                userCreatedStatus = UserController.CreateUser(ref newUser);
                if (userCreatedStatus == UserCreateStatus.Success)
                {
                    var objRoles = new RoleController();
                    RoleInfo objRole;

                    // autoassign user to portal roles
                    var arrRoles = objRoles.GetPortalRoles(newUser.PortalID);
                    for (int i = 0; i < arrRoles.Count - 1; i++)
                    {
                        objRole = (RoleInfo)arrRoles[i];
                        if (objRole.AutoAssignment)
                        {
                            objRoles.AddUserRole(newUser.PortalID, newUser.UserID, objRole.RoleID, DateTime.MinValue, DateTime.MaxValue);
                        }
                    }

                    // assign user to specified roles
                    if (roles != null)
                    {
                        foreach (string rolename in roles)
                        {
                            AddRoleToUser(portalId, newUser, rolename, DateTime.MaxValue);
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                DotNetNuke.Services.Exceptions.Exceptions.LogException(ex);
            }

            return userCreatedStatus;
        }

        /// <summary>
        /// AddRoleToUser
        /// </summary>
        /// <param name="user"></param>
        /// <param name="roleName"></param>
        /// <param name="expiry"></param>
        /// <returns></returns>
        public static bool AddRoleToUser(int portalid, UserInfo user, string roleName, DateTime expiry)
        {
            bool rc = false;
            if (user != null)
            {
                var roleCtl = new RoleController();
                RoleInfo newRole = roleCtl.GetRoleByName(portalid, roleName);
                if (newRole != null)
                {
                    roleCtl.AddUserRole(portalid, user.UserID, newRole.RoleID, DateTime.MinValue, expiry);
                    // Refresh user and check if role was added
                    user = UserController.GetUserById(portalid, user.UserID);
                    rc = user.IsInRole(roleName);
                }
            }
            return rc;
        }

        /// <summary>
        /// RemoveUserRole
        /// </summary>
        /// <param name="user"></param>
        /// <param name="delRole"></param>
        public static void RemoveUserRole(UserInfo user, RoleInfo delRole)
        {
            if (user != null && delRole != null)
            {
                if (user.IsInRole(delRole.RoleName))
                {
                    PortalController pCtrl = new PortalController();
                    foreach (PortalInfo p in pCtrl.GetPortals())
                    {
                        PortalSettings pSetting = new PortalSettings(p.PortalID);
                        RoleController.DeleteUserRole(user, delRole, pSetting, false);
                    }
                }
            }
        }

        /// <summary>
        /// CreateProfileProperty
        /// </summary>
        /// <param name="portalId"></param>
        /// <param name="propertyName"></param>
        /// <param name="propertyCategory"></param>
        /// <param name="type"></param>
        /// <param name="textLength"></param>
        /// <param name="visibility"></param>
        /// <param name="visible"></param>
        /// <returns></returns>
        public static int CreateProfileProperty(int portalId, string propertyName, string propertyCategory, ProfilePropertyDataTypes type, Int16 textLength, UserVisibilityMode visibility, bool visible)
        {
            var listController = new ListController();
            Dictionary<string, ListEntryInfo> dataTypes = listController.GetListEntryInfoDictionary("DataType");
            ListEntryInfo typeInfo = dataTypes["DataType:" + type.ToString()] ?? dataTypes["DataType:Unknown"];

            ProfilePropertyDefinition pdef = new ProfilePropertyDefinition(portalId);
            pdef.PropertyCategory = propertyCategory;
            pdef.PropertyName = propertyName;
            pdef.DefaultValue = string.Empty;
            pdef.ProfileVisibility = new ProfileVisibility { VisibilityMode = visibility };
            pdef.Visible = visible;
            pdef.DataType = typeInfo.EntryID; // 349=text, etc
            pdef.Length = textLength;

            return ProfileController.AddPropertyDefinition(pdef);
        }

        /// <summary>
        /// UpdateProfileProperty
        /// </summary>
        /// <param name="user"></param>
        /// <param name="profileField"></param>
        /// <param name="value"></param>
        public static void UpdateProfileProperty<T>(ref UserInfo user, string profileField, T value)
        {
            int pdefid = -1;
            ProfilePropertyDefinition pdef = ProfileController.GetPropertyDefinitionByName(user.PortalID, profileField);
            if (pdef == null)
            {
                ProfilePropertyDataTypes dataType = ProfilePropertyDataTypes.Text;
                if (typeof(T) == typeof(Int32))
                    dataType = ProfilePropertyDataTypes.Integer;
                else if (typeof(T) == typeof(DateTime))
                    dataType = ProfilePropertyDataTypes.DateTime;
                else if (typeof(T) == typeof(Boolean))
                    dataType = ProfilePropertyDataTypes.TrueFalse;

                ProfileVisibility v = new ProfileVisibility();
                v.VisibilityMode = UserVisibilityMode.AdminOnly;
                pdefid = CreateProfileProperty(user.PortalID, profileField, "Reddit",
                                                dataType, 0, UserVisibilityMode.AdminOnly, true);
            }
            else
            {
                pdefid = pdef.PropertyDefinitionId;
            }

            if (pdefid > 0)
            {
                user.Profile.SetProfileProperty(profileField, value.ToString());
                ProfileController.UpdateUserProfile(user);
            }
        }

        /// <summary>
        /// </summary>
        public static string PasswordStrengthInit(string passwordClientId)
        {
            string script = string.Empty;
            if (Host.EnableStrengthMeter)
            {
                var options = new DnnPaswordStrengthOptions();
                var optionsAsJsonString = Json.Serialize(options);
                script = string.Format("dnn.initializePasswordStrength('#{0}', {1});{2}",
                    passwordClientId, optionsAsJsonString, Environment.NewLine);
            }
            return script;
        }

        /// <summary>
        /// GetUserCreateStatusMessage
        /// </summary>
        /// <param name="status"></param>
        /// <returns></returns>
        public static string GetUserCreateStatusMessage(UserCreateStatus status, ref ModuleMessage.ModuleMessageType msgType)
        {
            string statusMsg;
            msgType = ModuleMessage.ModuleMessageType.RedError;
            switch (status)
            {
                case UserCreateStatus.AddUser:
                    statusMsg = "User created successfully";
                    msgType = ModuleMessage.ModuleMessageType.GreenSuccess;
                    break;
                case UserCreateStatus.UsernameAlreadyExists:
                    statusMsg = "Username created with this email address already exists";
                    msgType = ModuleMessage.ModuleMessageType.YellowWarning;
                    break;
                case UserCreateStatus.UserAlreadyRegistered:
                    statusMsg = "Username already been registered";
                    break;
                case UserCreateStatus.DuplicateEmail:
                    statusMsg = "A user with the same email address has already registered";
                    msgType = ModuleMessage.ModuleMessageType.YellowWarning;
                    break;
                case UserCreateStatus.DuplicateProviderUserKey:
                    statusMsg = "The authentication provider is trying to add a duplicate user";
                    break;
                case UserCreateStatus.DuplicateUserName:
                    statusMsg = "The authentication provider is trying to add a duplicate user name";
                    break;
                case UserCreateStatus.InvalidAnswer:
                    statusMsg = "You entered an invalid password request answer";
                    break;
                case UserCreateStatus.InvalidEmail:
                    statusMsg = "Email address for user account is invalid";
                    break;
                case UserCreateStatus.InvalidPassword:
                    statusMsg = "Password is not the correct length or strength";
                    break;
                case UserCreateStatus.InvalidProviderUserKey:
                    statusMsg = "The authentication provider is trying to add a user using an invalid key";
                    break;
                case UserCreateStatus.InvalidQuestion:
                    statusMsg = "You entered an invalid password request question";
                    break;
                case UserCreateStatus.InvalidUserName:
                    statusMsg = "The username requested is not the correct length or has invalid characters";
                    break;
                case UserCreateStatus.ProviderError:
                    statusMsg = "An authentication provider error occurred while trying to create the user";
                    break;
                case UserCreateStatus.Success:
                    statusMsg = "User created successfully";
                    msgType = ModuleMessage.ModuleMessageType.GreenSuccess;
                    break;
                case UserCreateStatus.UserRejected:
                    statusMsg = "User membership was rejected by administrator";
                    break;
                case UserCreateStatus.PasswordMismatch:
                    statusMsg = "The passwords entered do not match";
                    break;
                case UserCreateStatus.AddUserToPortal:
                    statusMsg = "User was added to portal";
                    msgType = ModuleMessage.ModuleMessageType.GreenSuccess;
                    break;
                case UserCreateStatus.InvalidDisplayName:
                    statusMsg = "Display name entered is invalid";
                    break;
                case UserCreateStatus.DuplicateDisplayName:
                    statusMsg = "A user with the same display name already exists";
                    msgType = ModuleMessage.ModuleMessageType.YellowWarning;
                    break;
                default:
                    statusMsg = "An unexpected error occurred while trying to create the user";
                    break;
            }
            return statusMsg;
        }

        public enum ProfilePropertyDataTypes
        {
            Unknown,
            Text,
            Integer,
            TrueFalse,
            TimeZone,
            Locale,
            Page,
            RichText,
            Country,
            Region,
            List,
            Date,
            DateTime,
            Image,
            Checkbox,
            TimeZoneInfo
        }

    }
}
