﻿/*
' Copyright (c) 2014 EVAddicts
' http://www.evaddicts.com
' All rights reserved.
' 
' THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
' TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
' THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
' CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
' DEALINGS IN THE SOFTWARE.
' 
*/
using System;
using DotNetNuke.Entities.Modules;
using DotNetNuke.Services.Exceptions;

namespace DotNetNuclear.Modules.Registration.Views
{

    public partial class Settings : ModuleSettingsBase
    {
        #region Base Method Implementations

        /// -----------------------------------------------------------------------------
        /// <summary>
        /// LoadSettings loads the settings from the Database and displays them
        /// </summary>
        /// -----------------------------------------------------------------------------
        public override void LoadSettings()
        {
            try
            {
                if (Page.IsPostBack == false)
                {
                    Components.SettingsRepository settingsCtrl = new Components.SettingsRepository(this.TabModuleId);

                    chkRecaptchaEnable.Checked = settingsCtrl.ReCaptchaEnabled;
                    txtReCaptchaPublicKey.Text = settingsCtrl.ReCaptchaPublicKey;
                    txtReCaptchaPrivateKey.Text = settingsCtrl.ReCaptchaPrivateKey;
                    ddlRecaptchaTheme.SelectedValue = settingsCtrl.ReCaptchaTheme;
                }
            }
            catch (Exception exc) //Module failed to load
            {
                Exceptions.ProcessModuleLoadException(this, exc);
            }
        }

        /// -----------------------------------------------------------------------------
        /// <summary>
        /// UpdateSettings saves the modified settings to the Database
        /// </summary>
        /// -----------------------------------------------------------------------------
        public override void UpdateSettings()
        {
            try
            {
                Components.SettingsRepository settingsCtrl = new Components.SettingsRepository(this.TabModuleId);

                settingsCtrl.ReCaptchaEnabled = chkRecaptchaEnable.Checked;
                settingsCtrl.ReCaptchaPublicKey = txtReCaptchaPublicKey.Text;
                settingsCtrl.ReCaptchaPrivateKey = txtReCaptchaPrivateKey.Text;
                settingsCtrl.ReCaptchaTheme = ddlRecaptchaTheme.SelectedValue;
            }
            catch (Exception exc) //Module failed to load
            {
                Exceptions.ProcessModuleLoadException(this, exc);
            }
        }

        #endregion
    }
}