﻿/*
' Copyright (c) 2014 EVAddicts
' http://www.evaddicts.com
' All rights reserved.
' 
' THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
' TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
' THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
' CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
' DEALINGS IN THE SOFTWARE.
' 
*/
using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using System.Web.UI.WebControls;
using DotNetNuke.Security;
using DotNetNuke.Services.Exceptions;
using DotNetNuke.Entities.Modules;
using DotNetNuke.Entities.Modules.Actions;
using DotNetNuke.Services.Localization;
using DotNetNuke.UI.Utilities;
using DotNetNuke.Web.Mvp;
using DotNetNuke.Web.Client.ClientResourceManagement;

using DotNetNuclear.Modules.Registration.Components;
using DotNetNuke.Entities.Users;
using DotNetNuke.Security.Membership;
using DotNetNuke.UI.Skins.Controls;
using DotNetNuke.Web.Client;
using System.Web;

namespace DotNetNuclear.Modules.Registration.Views
{
    public partial class View : PortalModuleBase
    {
        private int _portalRegType = 0;
        private SettingsRepository _settings;

        public string RegistrationMessage { get; set; }
        public ModuleMessage.ModuleMessageType RegistrationMessageStatus { get; set; }
        public string PasswordStrengthScript {
            get
            {
                return ViewState["PasswordStrengthScript"].ToString();
            }
            set
            {
                ViewState["PasswordStrengthScript"] = value;
            }
        }


        /// <summary>
        /// </summary>
        protected void Page_Load(object sender, EventArgs e)
        {
            try
            {
                _settings = new SettingsRepository(this.ModuleContext.TabModuleId);
                _portalRegType = base.ModuleContext.PortalSettings.UserRegistration;

                recaptcha.Theme = _settings.ReCaptchaTheme;
                recaptcha.PublicKey = _settings.ReCaptchaPublicKey;
                recaptcha.PrivateKey = _settings.ReCaptchaPrivateKey;
                recaptcha.Visible = _settings.ReCaptchaEnabled &&
                            (!String.IsNullOrEmpty(_settings.ReCaptchaPublicKey) && !String.IsNullOrEmpty(_settings.ReCaptchaPrivateKey));

                if (!IsPostBack)
                {
                    //ddlCountry
                    ddlCountry.Items.Clear();
                    DotNetNuke.Common.Lists.ListController lc = new DotNetNuke.Common.Lists.ListController();
                    ddlCountry.DataSource = lc.GetListEntryInfoItems("Country", "", base.ModuleContext.PortalSettings.PortalId);
                    ddlCountry.DataBind();
                    if (base.PortalSettings.CultureCode == "en-US")
                    {
                        ddlCountry.SelectedValue = "US";
                        ddlCountry_SelectedIndexChanged(sender, e);
                    }

                    PasswordStrengthScript = DnnMembershipUtilities.PasswordStrengthInit(txtPassword.ClientID);
                }
            }
            catch (Exception exc) //Module failed to load
            {
                Exceptions.ProcessModuleLoadException(this, exc);
            }
        }

        /// <summary>
        /// </summary>
        protected void Page_PreRender(object sender, EventArgs e)
        {
            ClientResourceManager.RegisterStyleSheet(this.Page, "~/DesktopModules/Admin/Security/module.css");
            ClientResourceManager.RegisterStyleSheet(this.Page, this.ControlPath + "../Resources/css/view.css");
            ClientResourceManager.RegisterScript(this.Page, this.ControlPath + "../Resources/js/jquery.validate.min.js", FileOrder.Js.jQueryUI);
            ClientResourceManager.RegisterScript(this.Page, "~/Resources/Shared/scripts/dnn.PasswordStrength.js", FileOrder.Js.jQueryUI);
        }

        protected void ddlCountry_SelectedIndexChanged(object sender, EventArgs e)
        {
            String code = ddlCountry.SelectedValue;
            DotNetNuke.Common.Lists.ListController lc = new DotNetNuke.Common.Lists.ListController();
            var stateList = lc.GetListEntryInfoItems("Region", "Country." + code, base.ModuleContext.PortalSettings.PortalId);
            ddlRegion.DataSource = stateList;
            ddlRegion.DataBind();
            ddlRegion.Visible = stateList.Any();
            txtRegion.Visible = !ddlRegion.Visible;
        }

        protected void btnRegister_Click(object sender, EventArgs e)
        {
            if (Page.IsValid)
            {
                // Create user model from form values
                UserInfo RegisterUserInfo = new UserInfo
                {
                    PortalID = ModuleContext.PortalId,
                    IsSuperUser = false,
                    FirstName = txtFirstName.Text.Trim(),
                    LastName = txtLastName.Text.Trim(),
                    LastIPAddress = HttpContext.Current.Request.UserHostAddress,
                    Username = txtUsername.Text.Trim(),
                    Email = txtEmail.Text.Trim()
                };

                RegisterUserInfo.Profile.Country = ddlCountry.SelectedItem.Text;
                RegisterUserInfo.Profile.Region = (String.IsNullOrEmpty(txtRegion.Text.Trim()) ? ddlRegion.SelectedItem.Text : txtRegion.Text.Trim());
                RegisterUserInfo.Profile.City = txtCity.Text.Trim();
                RegisterUserInfo.Profile.PostalCode = txtZip.Text.Trim();

                UserMembership oNewMembership = new UserMembership(RegisterUserInfo) { Password = txtPassword.Text };
                RegisterUserInfo.Membership = oNewMembership;

                // Register User
                UserCreateStatus status = DnnMembershipUtilities.CreateDnnUser(this.PortalId, ref RegisterUserInfo, null);
                if (status != UserCreateStatus.Success)
                {
                    ModuleMessage.ModuleMessageType mType = ModuleMessage.ModuleMessageType.YellowWarning;
                    RegistrationMessage = DnnMembershipUtilities.GetUserCreateStatusMessage(status, ref mType);
                    RegistrationMessageStatus = mType;
                }

                // Display any errors
                if (!String.IsNullOrEmpty(RegistrationMessage))
                {
                    DotNetNuke.UI.Skins.Skin.AddModuleMessage(this, RegistrationMessage, RegistrationMessageStatus);
                }
                else
                {
                    // Show success
                    DotNetNuke.UI.Skins.Skin.AddModuleMessage(this, LocalizeString("UserCreateSuccess.Text"), ModuleMessage.ModuleMessageType.GreenSuccess);
                    clearForm();
                }
            }
            else
            {
                if (!recaptcha.IsValid)
                    DotNetNuke.UI.Skins.Skin.AddModuleMessage(this, LocalizeString("ValidationCaptchaReq.Text"), ModuleMessage.ModuleMessageType.YellowWarning);
                else
                    DotNetNuke.UI.Skins.Skin.AddModuleMessage(this, LocalizeString("ValidationGeneralError.Text"), ModuleMessage.ModuleMessageType.RedError);
            }
        }

        private void clearForm()
        {
            txtFirstName.Text = "";
            txtLastName.Text = "";
            ddlCountry.SelectedIndex = 0;
            txtCity.Text = "";
            txtRegion.Text = "";
            txtZip.Text = "";
            txtEmail.Text = "";
            txtUsername.Text = "";
            txtPassword.Text = "";
            txtPasswordConfirm.Text = "";
            chkTermsAgreement.Checked = false;
        }

    }
}